@@ -11,7 +11,8 @@ import os.path
 import os
 import numpy as np
 import cv2 as cv
-from numpy.lib.arraysetops import isin
+
+# from numpy.lib.arraysetops import isin
 from machinevisiontoolbox.base import (
     int_image,
     float_image,
@@ -40,6 +41,7 @@ from machinevisiontoolbox.ImageReshape import ImageReshapeMixin
 from machinevisiontoolbox.ImageWholeFeatures import ImageWholeFeaturesMixin
 from machinevisiontoolbox.ImageBlobs import ImageBlobsMixin
 from machinevisiontoolbox.ImageRegionFeatures import ImageRegionFeaturesMixin
+from machinevisiontoolbox.ImageFiducials import ImageFiducialsMixin
 from machinevisiontoolbox.ImageLineFeatures import ImageLineFeaturesMixin
 from machinevisiontoolbox.ImagePointFeatures import ImagePointFeaturesMixin
 from machinevisiontoolbox.ImageMultiview import ImageMultiviewMixin
@@ -61,6 +63,7 @@ class Image(
     ImageBlobsMixin,
     ImageWholeFeaturesMixin,
     ImageRegionFeaturesMixin,
+    ImageFiducialsMixin,
     ImageLineFeaturesMixin,
     ImagePointFeaturesMixin,
     ImageMultiviewMixin,
@@ -307,7 +310,7 @@ class Image(
         """
         s = f"Image: {self.width} x {self.height} ({self.dtype})"
 
-        if self.iscolor:
+        if self.colororder is not None:
             s += ", " + self.colororder_str
         if self.id is not None:
             s += f", id={self.id}"
@@ -354,7 +357,8 @@ class Image(
         :note:
             - For a boolean image True and False are displayed as 1 and 0
               respectively.
-            - For a multiplane images the planes are printed sequentially.
+            - For a multiplane images the planes are printed sequentially, along
+              with the plane's name.
 
         :seealso: :meth:`Image.strhcat` :meth:`Image.showpixels`
         """
@@ -564,7 +568,6 @@ class Image(
 
     @colororder.setter
     def colororder(self, colororder: str | dict[str, int]) -> None:
-
         cdict = Image.colordict(colororder)
 
         if len(cdict) != self.nplanes:
@@ -596,6 +599,8 @@ class Image(
             >>> Image.colordict('RGB')
             >>> Image.colordict('red:green:blue')
             >>> Image.colordict({'L*': 0, 'U*': 1, 'V*': 2})
+
+        :deprecated: use :meth:`colororder2dict`
         """
         if isinstance(colororder, dict):
             cdict = colororder
@@ -612,6 +617,103 @@ class Image(
             raise ValueError("color order must be a dict or string")
         return cdict
 
+    @staticmethod
+    def colororder2dict(colororder: str, start=0) -> dict[str, int]:
+        """
+        Parse a color order specification to a color dictionary
+
+        :param colororder: order of color channels
+        :type colororder: str, dict
+        :raises ValueError: ``colororder`` not a string or dict
+        :return: dictionary mapping color names to plane indices
+        :rtype: dict
+
+        The color order the value can be given in a variety of forms:
+
+        * simple string, one plane per character, eg. ``"RGB"``
+        * colon separated string, eg. ``"R:G:B"``, ``"L*:a*:b*"``
+        * dictionary, eg. ``dict(R=0, G=1, B=2)``
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from machinevisiontoolbox import Image
+            >>> Image.colororder2dict('RGB')
+            >>> Image.colororder2dict('red:green:blue')
+            >>> Image.colororder2dict({'L*': 0, 'U*': 1, 'V*': 2})
+        """
+        if isinstance(colororder, dict):
+            # already a dict
+            if start > 0:
+                # need to renumber the planes
+                cdict = {}
+                for key, value in colororder.items():
+                    cdict[key] = value + start
+                return cdict
+            else:
+                return colororder
+        elif isinstance(colororder, str):
+            if ":" in colororder:
+                colororder = colororder.split(":")
+            else:
+                colororder = list(colororder)
+
+            cdict = {}
+            for i, color in enumerate(colororder):
+                cdict[color] = i + start
+            return cdict
+        else:
+            raise ValueError("color order must be a dict or string")
+
+    @staticmethod
+    def colordict2list(cdict: dict[str, int]) -> list:
+        """
+        Convert a color dictionary to a list of color plane names
+
+        :param cdict: dictionary mapping color names to plane indices
+        :type cdict: dict
+        :return: list of color channels in plane order
+        :rtype: list
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from machinevisiontoolbox import Image
+            >>> Image.colordict2list({'R': 0, 'G': 1, 'B': 2})
+
+        :note: The color planes are sorted by their index value.  There is no
+            check that the lowest plane index is zero.
+
+        :seealso: :meth:`colordict2str` :meth:`colororder2dict`
+        """
+        return [x[0] for x in sorted(cdict.items(), key=lambda x: x[1])]
+
+    @staticmethod
+    def colordict2str(cdict: dict[str, int]) -> str:
+        """
+        Convert a color dictionary to a color order string
+
+        :param cdict: dictionary mapping color names to plane indices
+        :type cdict: dict
+        :return: color channel names in plane order as a colon-separated string
+        :rtype: str
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from machinevisiontoolbox import Image
+            >>> Image.colordict2list({'R': 0, 'G': 1, 'B': 2})
+
+        :note: The color planes are sorted by their index value.  There is no
+            check that the lowest plane index is zero.
+
+        :seealso: :meth:`colordict2list` :meth:`colororder2dict`
+        """
+        return ":".join(Image.colordict2list(cdict))
+
     @property
     def colororder_str(self) -> str:
         """
@@ -1728,23 +1830,24 @@ class Image(
             >>> red_blue = img.plane([0, 2]) # blue and red planes
             >>> red_blue
 
-        :note:
+        .. note::
+
             - This can also be performed using the overloaded ``__getitem__``
               operator.
             - To select more than one plane, use either a sequence of integers or a string
               of colon separated plane names.
+            - For a single-plane image the index must be zero.
 
         :seealso: :meth:`red` :meth:`green` :meth:`blue` :meth:``__getitem__``
         """
-        if not self.iscolor:
-            raise ValueError("cannot extract color plane from greyscale image")
 
         if isinstance(planes, int):
+            if planes == 0 and self.nplanes == 1:
+                return self
             if planes < 0 or planes >= self.nplanes:
                 raise ValueError("plane index out of range")
-            iplanes = planes
+            iplanes = [planes]
             colororder = None
-            planes = [planes]
         elif isinstance(planes, str):
             iplanes = []
             colororder = {}
@@ -2107,7 +2210,6 @@ class Image(
         return self._binop(self, other, lambda x, y: x * y)
 
     def __rmul__(self, other) -> "Image":
-
         return self._binop(self, other, lambda x, y: y * x)
 
     def __pow__(self, other) -> "Image":
@@ -2694,239 +2796,6 @@ class Image(
         """
         return self._unop(self, np.sqrt)
 
-    def sum(self, *args, **kwargs) -> "Image":
-        r"""
-        Sum of all pixels
-
-        :param args: additional positional arguments to :func:`numpy.sum`
-        :param kwargs: additional keyword arguments to :func:`numpy.sum`
-        :return: sum
-
-        Computes the sum of pixels in the image:
-
-        .. math::
-
-            \sum_{uvc} I_{uvc}
-
-        Example:
-
-        .. runblock:: pycon
-
-            >>> from machinevisiontoolbox import Image
-            >>> img = Image.Read('flowers1.png')
-            >>> img.sum()  # R+G+B
-            >>> img.sum(axis=(0,1)) # sum(R), sum(G), sum(B)
-            >>> img = Image.Read('flowers1.png', dtype='float32')
-            >>> img.sum(axis=2)
-
-
-        :note:
-            - The return value type is the same as the image type.
-            - By default the result is a scalar computed over all pixels,
-              if the ``axis`` option is given the results is a 1D or 2D NumPy
-              array.
-
-        :seealso: :func:`numpy.sum` :meth:`~~machinevisiontoolbox.ImageWholeFeatures.ImageWholeFeaturesMixin.mpq`
-            :meth:`~machinevisiontoolbox.ImageWholeFeatures.ImageWholeFeaturesMixin.npq`
-            :meth:`~machinevisiontoolbox.ImageWholeFeatures.ImageWholeFeaturesMixin.upq`
-        """
-        return np.sum(self.A, *args, **kwargs)
-
-    def min(self, *args, **kwargs) -> int | float:
-        """
-        Minimum value of all pixels
-
-        :param args: additional positional arguments to :func:`numpy.min`
-        :param kwargs: additional keyword arguments to :func:`numpy.min`
-        :return: minimum value
-
-        Example:
-
-        .. runblock:: pycon
-
-            >>> from machinevisiontoolbox import Image
-            >>> img = Image.Read('flowers1.png')
-            >>> img.min()
-            >>> img = Image.Read('flowers1.png', dtype='float32')
-            >>> img.min(axis=2)
-
-        :note:
-            - The return value type is the same as the image type.
-            - By default the result is a scalar computed over all pixels,
-              if the ``axis`` option is given the results is a 1D or 2D NumPy
-              array.
-
-        :seealso: :func:`numpy.min`
-        """
-        return np.min(self.A, *args, **kwargs)
-
-    def max(self, *args, **kwargs) -> int | float:
-        """
-        Maximum value of all pixels
-
-        :param args: additional positional arguments to :func:`numpy.max`
-        :param kwargs: additional keyword arguments to :func:`numpy.max`
-        :return: maximum value
-
-        Example:
-
-        .. runblock:: pycon
-
-            >>> from machinevisiontoolbox import Image
-            >>> img = Image.Read('flowers1.png')
-            >>> img.max()
-            >>> img = Image.Read('flowers1.png', dtype='float32')
-            >>> img.max(axis=2)
-
-        :note:
-            - The return value type is the same as the image type.
-            - By default the result is a scalar computed over all pixels,
-              if the ``axis`` option is given the results is a 1D or 2D NumPy
-              array.
-
-        :seealso: :func:`numpy.max`
-        """
-        return np.max(self.A, *args, **kwargs)
-
-    def mean(self, *args, **kwargs) -> float:
-        """
-        Mean value of all pixels
-
-        :param args: additional positional arguments to :func:`numpy.mean`
-        :param kwargs: additional keyword arguments to :func:`numpy.mean`
-        :return: mean value
-
-        Example:
-
-        .. runblock:: pycon
-
-            >>> from machinevisiontoolbox import Image
-            >>> img = Image.Read('flowers1.png')
-            >>> img.mean()
-            >>> img = Image.Read('flowers1.png', dtype='float32')
-            >>> img.mean(axis=2)
-
-        :note:
-            - The return value type is the same as the image type.
-            - By default the result is a scalar computed over all pixels,
-              if the ``axis`` option is given the results is a 1D or 2D NumPy
-              array.
-
-        :seealso: :func:`numpy.mean`
-        """
-        return np.mean(self.A, *args, **kwargs)
-
-    def std(self, *args, **kwargs) -> float:
-        """
-        Standard deviation of all pixels
-
-        :param args: additional positional arguments to :func:`numpy.std`
-        :param kwargs: additional keyword arguments to :func:`numpy.std`
-        :return: standard deviation value
-
-        Example:
-
-        .. runblock:: pycon
-
-            >>> from machinevisiontoolbox import Image
-            >>> img = Image.Read('flowers1.png')
-            >>> img.std()
-            >>> img = Image.Read('flowers1.png', dtype='float32')
-            >>> img.std()
-
-        :note:
-            - The return value type is the same as the image type.
-            - By default the result is a scalar computed over all pixels,
-              if the ``axis`` option is given the results is a 1D or 2D NumPy
-              array.
-
-        :seealso: :func:`numpy.std`
-        """
-        return np.std(self.A, *args, **kwargs)
-
-    def var(self, *args, **kwargs) -> float:
-        """
-        Variance of all pixels
-
-        :param args: additional positional arguments to :func:`numpy.var`
-        :param kwargs: additional keyword arguments to :func:`numpy.var`
-        :return: variance value
-
-        Example:
-
-        .. runblock:: pycon
-
-            >>> from machinevisiontoolbox import Image
-            >>> img = Image.Read('flowers1.png')
-            >>> img.var()
-            >>> img = Image.Read('flowers1.png', dtype='float32')
-            >>> img.var()
-
-        :note:
-            - The return value type is the same as the image type.
-            - By default the result is a scalar computed over all pixels,
-              if the ``axis`` option is given the results is a 1D or 2D NumPy
-              array.
-
-        :seealso: :func:`numpy.var`
-        """
-        return np.std(self.A, *args, **kwargs)
-
-    def median(self, *args, **kwargs) -> int | float:
-        """
-        Median value of all pixels
-
-        :param args: additional positional arguments to :func:`numpy.median`
-        :param kwargs: additional keyword arguments to :func:`numpy.median`
-        :return: median value
-
-        Example:
-
-        .. runblock:: pycon
-
-            >>> from machinevisiontoolbox import Image
-            >>> img = Image.Read('flowers1.png')
-            >>> img.median()
-            >>> img = Image.Read('flowers1.png', dtype='float32')
-            >>> img.median()
-
-        :note:
-            - The return value type is the same as the image type.
-            - By default the result is a scalar computed over all pixels,
-              if the ``axis`` option is given the results is a 1D or 2D NumPy
-              array.
-
-        :seealso: :func:`numpy.median`
-        """
-        return np.median(self.A, *args, **kwargs)
-
-    def stats(self) -> None:
-        """
-        Display pixel value statistics
-
-        Example:
-
-        .. runblock:: pycon
-
-            >>> from machinevisiontoolbox import Image
-            >>> img = Image.Read('flowers1.png')
-            >>> img.stats()
-        """
-
-        def printstats(plane):
-            print(
-                f"range={plane.min()} - {plane.max()}, "
-                f"mean={plane.mean():.3f}, "
-                f"sdev={plane.std():.3f}"
-            )
-
-        if self.iscolor:
-            for k, v in sorted(self.colororder.items(), key=lambda x: x[1]):
-                print(f"{k:s}: ", end="")
-                printstats(self.A[..., v])
-        else:
-            printstats(self.A)
-
     # ---------------------------- graphics ---------------------------- #
 
     def draw_line(self, start, end, **kwargs):
@@ -3010,10 +2879,70 @@ class Image(
         """
         draw_box(self.image, **kwargs)
 
+    @classmethod
+    def Pstack(cls, images, colororder=None):
+        """
+        Concatenation of image planes
+
+        :param images: images to concatenate plane-wise
+        :type images: iterable of :class:`Image`
+        :raises ValueError: all images must have the same dtype
+        :raises ValueError: all images must have the same color order
+        :return: horizontally stacked images
+        :rtype: :class:`Image`
+
+        Create a new image by stacking the input images horizontally, with a
+        vertical separator line of width ``sep`` and color ``bgcolor``.
+
+        The horizontal coordinate of the first column of each image, in the
+        composite output image, can be optionally returned if ``return_offsets``
+        is True.
+
+        Example:
+
+        .. runblock:: pycon
+
+            >>> from machinevisiontoolbox import Image
+            >>> img = Image.Read('street.png')
+            >>> img
+            >>> Image.Hstack((img, img, img))
+            >>> Image.Hstack((img, img, img), return_offsets=True)
+
+        .. plot::
+
+            from machinevisiontoolbox import Image
+            img = Image.Read('street.png')
+            Image.Hstack((img, img, img)).disp()
+
+
+        :seealso: :meth:`Vstack` :meth:`Hstack`
+        """
+        nplanes = images[0].nplanes
+        for image in images[1:]:
+            if image.dtype != images[0].dtype:
+                raise ValueError("all planes must have the same dtype")
+            if image.size != images[0].size:
+                raise ValueError("all planes must have the same width x height")
+            nplanes += image.nplanes
+
+        if colororder is None:
+            # attempt to create color order from the images
+            colororder = images[0].colororder
+            ip = len(colororder)
+            for image in images[1:]:
+                colororder |= Image.colororder2dict(image.colororder, start=ip)
+        else:
+            if len(Image.colororder2dict(colororder)) != nplanes:
+                raise ValueError("colororder does not match number of planes")
+
+        return Image(
+            np.concatenate([np.atleast_3d(im.A) for im in images], axis=2),
+            colororder=colororder,
+        )
+
 
 # --------------------------------------------------------------------------- #
 if __name__ == "__main__":
-
     import pathlib
     import os.path
     from machinevisiontoolbox import Image
